from django.contrib import admin
from django.conf import settings

from django.http import HttpResponse
from django.utils.text import capfirst
from django.template.defaultfilters import title, escape, force_escape, escapejs
from django.forms import CharField

from cms.models.pluginmodel import CMSPlugin
from cms.forms.widgets import PlaceholderPluginEditorWidget
from cms.admin.placeholderadmin import PlaceholderAdmin

def get_or_create_placeholders(obj, model):
    if obj:
        for placeholder_name in model._meta.get_field('placeholders').placeholders:
            placeholder, created = obj.placeholders.get_or_create(slot=placeholder_name)
            yield (placeholder, placeholder_name)
    else:
        for placeholder_name in model._meta.get_field('placeholders').placeholders:
            yield (None, placeholder_name)    
            
def get_m2mplaceholderadmin(modeladmin):
    class RealM2MPlaceholderAdmin(modeladmin):
        
        def get_form(self, request, obj=None, **kwargs):
            """
            Get PageForm for the Page model and modify its fields depending on
            the request.
            """
            form = super(RealM2MPlaceholderAdmin, self).get_form(request, obj, **kwargs)
            
            for placeholder, slot in get_or_create_placeholders(obj, self.model):

                defaults = {'label': capfirst(slot), 'help_text': ''}
                defaults.update(kwargs)
                    
                widget = PlaceholderPluginEditorWidget(request, self.placeholder_plugin_filter)
                widget.choices = []
                    
                form.base_fields[slot] = CharField(widget=widget, required=False)
                
                if placeholder:   
                    form.base_fields[slot].initial = placeholder.pk
                    
            return form
            
        def get_fieldsets(self, request, obj=None):
            """
            Add fieldsets of placeholders to the list of already existing
            fieldsets.
            """
            given_fieldsets = super(RealM2MPlaceholderAdmin, self).get_fieldsets(request, obj=None)

            for placeholder_name in self.model._meta.get_field('placeholders').placeholders:
                given_fieldsets += [(title(placeholder_name), {'fields':[placeholder_name], 'classes':['plugin-holder']})]
    
            return given_fieldsets
                
        def move_plugin(self, request):

            if request.method == "POST":    
                if 'plugin_id' in request.POST:
                    plugin = CMSPlugin.objects.get(pk=int(request.POST['plugin_id']))
                    if "placeholder" in request.POST:
                        obj = plugin.placeholder._get_attached_model().objects.get(placeholders__cmsplugin=plugin) # FIXME only works if field is named placeholers
                        placeholder = obj.placeholders.get(slot=request.POST["placeholder"])
                    else:
                        placeholder = plugin.placeholder
                    # plugin positions are 0 based, so just using count here should give us 'last_position + 1'
                    position = CMSPlugin.objects.filter(placeholder=placeholder).count()
                    plugin.placeholder = placeholder
                    plugin.position = position
                    plugin.save()
                pos = 0
                if 'ids' in request.POST:
                    for id in request.POST['ids'].split("_"):
                        plugin = CMSPlugin.objects.get(pk=id)
                        if plugin.position != pos:
                            plugin.position = pos
                            plugin.save()
                        pos += 1
                else:
                    HttpResponse(str("error"))
                return HttpResponse(str("ok"))
            else:
                return HttpResponse(str("error"))
    return RealM2MPlaceholderAdmin

M2MPlaceholderAdmin = get_m2mplaceholderadmin(PlaceholderAdmin)


from django import template
from classytags.arguments import Argument
from classytags.parser import Parser
from classytags.core import Options
from cms.templatetags.placeholder_tags import RenderPlaceholder
from cms.models import Placeholder
register = template.Library()
 
class RenderPlaceholderAs(RenderPlaceholder):
    name = 'render_placeholder_as'
    options = Options(
        Argument('placeholder'),
        'as',
        Argument('as_var', default=None, required=False),
        Argument('width', default=None, required=False)
    )

    def render_tag(self, context, **kwargs):
        rendered = super(RenderPlaceholderAs, self).render_tag(context, kwargs['placeholder'], kwargs['width'])
        if kwargs['as_var']:
            context.push()
            context[str(kwargs['as_var'])] = rendered
            return u''
        return rendered

register.tag(RenderPlaceholderAs)

def choose_placeholder(placeholders, placeholder):
    try:
        return placeholders.get(slot=placeholder)
    except Placeholder.DoesNotExist:
        return None
register.filter('choose_placeholder', choose_placeholder)

def get_placeholder_group(placeholder):
    placeholder_group = 'none'
    for group, placeholders in settings.PLACEHOLDER_GROUPS.items():
        if placeholder in placeholders:
            placeholder_group = group
    return placeholder_group
register.filter('get_placeholder_group', get_placeholder_group)



import unittest
from django.test.testcases import TestCase
from django.core.handlers.wsgi import WSGIRequest
from django.conf import settings
from django.template import Template, Context
from cms.plugins.text.models import Text
from djangocms_utils.test.testapp.models import MultiplePlaceholdersExample, PlaceholderAsExample
from djangocms_utils.admin import get_or_create_placeholders
from djangocms_utils.templatetags.djangocms_utils_tags import choose_placeholder

class MultiplePlaceholdersTestCase(unittest.TestCase):
    
    def setUp(self):
        
        self.obj = MultiplePlaceholdersExample(heading='Example heading')
        self.obj.save()
        
        for placeholder, slot in get_or_create_placeholders(self.obj, MultiplePlaceholdersExample):
            for i in range(3):
                plugin = Text(plugin_type='TextPlugin',
                    placeholder=placeholder,
                    position=i,
                    body='Page %s' %(i),
                    language=settings.LANGUAGE_CODE
                )
                plugin.insert_at(None, position='last-child', commit=True)
                        
class TestAttachedField(MultiplePlaceholdersTestCase):
    
    def testAttachedField(self):
        # make sure manytomany fields are found as the field Placeholder is related via
        ph = self.obj.placeholders.all()[0]
        self.assertEqual('placeholders', ph._get_attached_field().name)
                
class ChoosePlaceholderTestCase(MultiplePlaceholdersTestCase):
    
    def testChoosePlaceholder(self):
        ph = self.obj.placeholders.get(slot='first')
        self.assertEqual(choose_placeholder(self.obj.placeholders, 'first'), ph)

class RenderPlaceholderAsTestCase(TestCase):
    
    def get_request(self, path=None):
        
        if not path:
            path = '/'

        environ = {
            'HTTP_COOKIE': self.client.cookies,
            'PATH_INFO': path,
            'QUERY_STRING': '',
            'REMOTE_ADDR': '127.0.0.1',
            'REQUEST_METHOD': 'GET',
            'SCRIPT_NAME': '',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': '80',
            'SERVER_PROTOCOL': 'HTTP/1.1',
            'wsgi.version': (1,0),
            'wsgi.url_scheme': 'http',
            'wsgi.errors': self.client.errors,
            'wsgi.multiprocess': True,
            'wsgi.multithread': False,
            'wsgi.run_once': False,
        }
        request = WSGIRequest(environ)
        request.session = self.client.session
        request.LANGUAGE_CODE = settings.LANGUAGE_CODE
        return request
        
    def setUp(self):
        
        self.obj = PlaceholderAsExample(heading='Example heading')
        self.obj.save()
        
        placeholder = self.obj.placeholder
        
        for i in range(3):
            plugin = Text(plugin_type='TextPlugin',
                placeholder=placeholder,
                position=i,
                body='Page %s' %(i),
                language=settings.LANGUAGE_CODE
            )
            plugin.insert_at(None, position='last-child', commit=True)
                        
    def testRenderAs(self):
        t = Template('{% load djangocms_utils_tags %}{% render_placeholder_as obj.placeholder as testcontent %}{{ testcontent }}')
        c = Context({"obj": self.obj, "request": self.get_request()})
        out = t.render(c)
        self.assertEqual(out, u'Page 0\nPage 1\nPage 2\n')






from setuptools import setup

setup(name='djangocms-utils',
      version='0.9.3',
      description='Utilities for django-cms',
      author='Oyvind Saltvik',
      author_email='oyvind.saltvik@gmail.com',
      url='http://github.com/fivethreeo/djangocms-utils/',
      packages = ['djangocms_utils', 'djangocms_utils.templatetags'],
      test_suite = 'djangocms_utils.test.run_tests.run_tests',
      install_requires = ['django-cms>=2.1.0']
)
